<?php

namespace Drupal\campaign_kit;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Link;
//Added
use Drupal\Core\Datetime\DrupalDateTime;

/**
 * Defines a class to build a listing of Campaign donation entities.
 *
 * @ingroup campaign_kit
 */
class CampaignDonationListBuilder extends EntityListBuilder {


  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['campaign_id'] = $this->t('Campaign ID');
    $header['campaign_name'] = $this->t('Campaign Name');
    //$header['id'] = $this->t('Campaign donation ID');
    //$header['name'] = $this->t('Name');
    $header['full_name'] = $this->t('Full Name');
    $header['frequency'] = $this->t('Frequency');
    $header['one_time_donation'] = $this->t('One-Time Donation');
    $header['monthly_donation'] = $this->t('Monthly Donation');
    $header['date_time'] = $this->t('Date');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    /* @var $entity \Drupal\campaign_kit\Entity\CampaignDonation */
    $row['campaign_id'] = $entity->getCampaign()->id();
    $row['campaign_name'] = Link::createFromRoute(
      $entity->getCampaign()->label(),
      'campaign_kit.campaign_kit_controller_view_campaign_entity',
      ['campaign' => $entity->getCampaign()->id()]
    );
    $row['full_name'] = Link::createFromRoute(
      $entity->label(),
      'entity.campaign_donation.edit_form',
      ['campaign_donation' => $entity->id()]
    );

    $row['frequency'] = $this->getFrequencyLabel($entity->getField('donation_frequency'));

    $legendCurrency = $this->getCurrency($entity->getField('currency'));
    // Check if it is ONE-TIME or MONTHLY
    if ($entity->getField('donation_frequency') == 'onetime') {
      $row['one_time_donation'] = $legendCurrency . ' ' . $entity->getField('amount');
      $row['monthly_donation'] = '';
    } else {
      $row['one_time_donation'] = '';
      $row['monthly_donation'] = $legendCurrency . ' ' . $entity->getField('amount');
    }

    $datetime = DrupalDateTime::createFromTimestamp($entity->getCreatedTime());
    $row['date_time'] = $datetime->format('Y-m-d H:i');
    return $row + parent::buildRow($entity);
  }

  /**
   * @param $currency
   *
   * @return string
   */
  private function getCurrency($currency) {
    switch ($currency) {
      case 'USD':
        return '$';
        break;
    }
  }

  /**
   * @param $frequency
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup|void
   */
  private function getFrequencyLabel($frequency) {
    switch ($frequency) {
      case 'onetime':
        return t('One-time');
        break;
      case 'monthly':
        return t('Monthly');
        break;
    }
  }

}
