<?php

namespace Drupal\campaign_kit;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Link;

/**
 * Defines a class to build a listing of Campaign entities.
 *
 * @ingroup campaign_kit
 */
class CampaignListBuilder extends EntityListBuilder {


  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['id'] = $this->t('Campaign ID');
    $header['name'] = $this->t('Name');
    $header['total_one_time'] = $this->t('Total One-Time');
    $header['total_monthly'] = $this->t('Total Monthly');
    //$header['total_amount'] = $this->t('Total Amount');
    $header['status'] = $this->t('Status');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    // Get Currency code
    $currencyCode = $this->getCurrency();

    /* @var $entity \Drupal\campaign_kit\Entity\Campaign */
    $row['id'] = $entity->id();
    $row['name'] = Link::createFromRoute(
      $entity->label(),
      //'entity.campaign.edit_form',
      'campaign_kit.campaign_kit_controller_view_campaign_entity',
      //['campaign' => $entity->id()]
      ['campaign' => $entity->id()]
    );
    $row['total_one_time'] = $currencyCode . ' ' . $entity->getRaisedAmountOneTime();
    $row['total_monthly'] = $currencyCode . ' ' . $entity->getRaisedAmountMonthly();
    //$row['total_amount'] = $currencyCode . ' ' . $entity->getRaisedAmount();
    $row['status'] = $entity->getCampaignStatus();
    return $row + parent::buildRow($entity);
  }

  /**
   * @return mixed
   */
  private function getCurrency() {
    // Get currency from plugin
    $config = \Drupal::config('campaign_kit.paymentprocessor');
    $clientPlugin =  $config->get('campaign_kit');

    switch ($clientPlugin['currency']) {
      case 'USD':
        return '$';
        break;
    }
  }

}
